@@ -12,6 +12,8 @@
 // OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 //
 
+#[cfg(feature = "chrono")]
+use chrono::{DateTime, TimeZone, Utc};
 use nom::error::ErrorKind;
 use nom::number::streaming::{be_i16, be_i32, be_u16, be_u32, be_u8};
 use nom::IResult;
@@ -46,3 +48,11 @@ pub(crate) fn pf32(input: &[u8]) -> IResult<&[u8], f32> {
     let b = (a as f32) * 1e-7;
     Ok((i, b))
 }
+
+#[cfg(feature = "chrono")]
+#[allow(dead_code)]
+pub(crate) fn pdt(input: &[u8]) -> IResult<&[u8], DateTime<Utc>> {
+    let (i, a): (&[u8], u32) = be_u32::<_, (_, ErrorKind)>(input).unwrap();
+    let dt = Utc.timestamp(a, 0);
+    Ok((i, dt))
+}
